﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Documents.Excel;
using System.Data;
using Infragistics.Windows.Chart;
using System.Diagnostics;
using System.Windows.Media.Media3D;
using System.Reflection;

namespace _3DColumnChart
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            List<Brush> brushes = new List<Brush>();
            foreach (PropertyInfo prop in typeof(Brushes).GetProperties())
            {
                brushes.Add((Brush)prop.GetValue(null, null));
            }           

            ChartViewModel vm = new ChartViewModel();
            this.DataContext = vm;
            int i = 0;
            foreach (Model model in vm.ChartContext.OrderBy(m => m.Source.Count))
            {
                Brush fill = brushes[i++];
                Series series = new Series();
                series.ChartType = ChartType.Column;
                series.Label = model.Name;
                series.DataSource = model.Source;
                series.Fill = fill;
                series.DataMapping = "Value=Price;Label=Indicator;ToolTip=.";
                chart.Series.Add(series);
            }
        }        
    }

    public class ChartViewModel
    {
        public ChartViewModel()
        {
            ChartContext = new ObservableCollection<Model>();

            Workbook dataSource = Workbook.Load("XamChartData.xlsx");
            for (int row = 1; row < dataSource.Worksheets[0].Rows.Count(); row++)
            {
                ChartContext.Add(
                    new Model
                    {
                        Name = dataSource.Worksheets[0].Rows[row].Cells[1].Value.ToString(),
                        Source = SourceData(dataSource.Worksheets[1], int.Parse(dataSource.Worksheets[0].Rows[row].Cells[0].Value.ToString()), dataSource.Worksheets[0].Rows[row].Cells[1].Value.ToString()),
                    });
            }

        }

        private ObservableCollection<Data> SourceData(Worksheet sheet, int regID, string model)
        {
            ObservableCollection<Data> data = new ObservableCollection<Data>();

            for (int row = 1; row < sheet.Rows.Count(); row++)
            {
                if (int.Parse(sheet.Rows[row].Cells[0].Value.ToString()) == regID)
                {
                    data.Add(new Data
                    {
                        Indicator = sheet.Rows[row].Cells[1].Value.ToString(),
                        Price = double.Parse(sheet.Rows[row].Cells[2].Value.ToString()),
                        Model = model
                    });
                }
            }

            return data;
        }

        public ObservableCollection<Model> ChartContext { get; set; }
    }

    public class Model
    {
        public string Name { get; set; }
        public ObservableCollection<Data> Source { get; set; }
    }

    public class Data
    {
        public string Indicator { get; set; }
        public double Price { get; set; }
        public string Model { get; set; }
    }

    public class UperLeftDownRightAxisConverter : IMultiValueConverter
    {
        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            double result1;
            double  result2;
            if (Double.TryParse(values[0].ToString(), out result1) && Double.TryParse(values[1].ToString(),out result2))
            {
                return new Vector3D((double)values[0], (double)values[1], 0);
            }
            return Binding.DoNothing;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class UperRightDownLeftAxisConverter : IMultiValueConverter
    {
        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            double result1;
            double result2;
            if (Double.TryParse(values[0].ToString(), out result1) && Double.TryParse(values[1].ToString(), out result2))
            {
                return new Vector3D((double)values[0], -(double)values[1], 0);
            }
            return Binding.DoNothing;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class RotateLeftCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            ((((parameter as XamChart).Transform3D as Transform3DGroup).Children[1] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle -= 2;
        }
    }

    public class RotateRightCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            ((((parameter as XamChart).Transform3D as Transform3DGroup).Children[1] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle += 2;
        }
    }

    public class RotateUpCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            ((((parameter as XamChart).Transform3D as Transform3DGroup).Children[0] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle -= 2;
        }
    }

    public class RotateDownCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            ((((parameter as XamChart).Transform3D as Transform3DGroup).Children[0] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle += 2;
        }
    }

    public class RotateUpLeftCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            ((((parameter as XamChart).Transform3D as Transform3DGroup).Children[2] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle -= 2;
        }
    }

    public class RotateDownRightCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            ((((parameter as XamChart).Transform3D as Transform3DGroup).Children[2] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle += 2;
        }
    }

    public class RotateUpRightCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            ((((parameter as XamChart).Transform3D as Transform3DGroup).Children[3] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle -= 2;
        }
    }

    public class RotateDownLeftCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            ((((parameter as XamChart).Transform3D as Transform3DGroup).Children[3] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle += 2;
        }
    }

    public class RotateCommand : ICommand
    {
        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;

        public void Execute(object parameter)
        {
            ((((parameter as XamChart).Transform3D as Transform3DGroup).Children[4] as RotateTransform3D).Rotation as AxisAngleRotation3D).Angle += 2;
        }
    }

}